/*
* Copyright (c) 2018-2019, Intel Corporation
*
* Permission is hereby granted, free of charge, to any person obtaining a
* copy of this software and associated documentation files (the "Software"),
* to deal in the Software without restriction, including without limitation
* the rights to use, copy, modify, merge, publish, distribute, sublicense,
* and/or sell copies of the Software, and to permit persons to whom the
* Software is furnished to do so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included
* in all copies or substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
* OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
* THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR
* OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
* ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
* OTHER DEALINGS IN THE SOFTWARE.
*/
//!
//! \file      cm_event_ex.h
//! \brief     Contains Class CmEventEx  definitions
//!
#pragma once

#include "cm_event_ex_base.h"

class CmEventEx : public CmEventExBase
{
public:
    CmEventEx(CM_HAL_STATE *state, uint32_t id, CmTracker *tracker):
        CmEventExBase(state, id, tracker),
        m_osSignalTriggered(false)
    {
    }

    virtual ~CmEventEx();
    
    virtual int32_t WaitForTaskFinished(uint32_t timeOutMs = CM_MAX_TIMEOUT_MS);

    virtual int32_t GetStatus(CM_STATUS &status);

    void SetTaskOsData(MOS_RESOURCE *resource, HANDLE handle);

protected:
    virtual void RleaseOsData();
    bool m_osSignalTriggered;
};
