/*===================== begin_copyright_notice ==================================

* Copyright (c) 2021, Intel Corporation
*
* Permission is hereby granted, free of charge, to any person obtaining a
* copy of this software and associated documentation files (the "Software"),
* to deal in the Software without restriction, including without limitation
* the rights to use, copy, modify, merge, publish, distribute, sublicense,
* and/or sell copies of the Software, and to permit persons to whom the
* Software is furnished to do so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included
* in all copies or substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
* OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
* THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR
* OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
* ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
* OTHER DEALINGS IN THE SOFTWARE.

======================= end_copyright_notice ==================================*/

//!
//! \file     mhw_render_xe_hp_base.h
//! \brief    Defines functions for constructing  render engine commands on Gen12_5_plus platforms
//!

#ifndef __mhw_render_xe_xpm_base_H__
#define __mhw_render_xe_xpm_base_H__

#include "mhw_render_generic.h"
#include "mhw_render_g12_X.h"
#include "mhw_mmio_g12.h"
#include "mhw_render_hwcmd_xe_hp_base.h"

struct MHW_STATE_COMPUTE_MODE_PARAMS
{
    MHW_STATE_COMPUTE_MODE_PARAMS() : enableLargeGrf(false) {}

    bool enableLargeGrf;
};

class MhwRenderInterfaceXe_Xpm_Base : public MhwRenderInterfaceG12
{
public:
    MhwRenderInterfaceXe_Xpm_Base(
        MhwMiInterface *   miInterface,
        PMOS_INTERFACE     osInterface,
        MEDIA_SYSTEM_INFO *gtSystemInfo,
        uint8_t            newStateHeapManagerRequested) : 
        MhwRenderInterfaceG12(miInterface, osInterface, gtSystemInfo, newStateHeapManagerRequested)
    {
        MHW_FUNCTION_ENTER;
    }

    virtual ~MhwRenderInterfaceXe_Xpm_Base() { MHW_FUNCTION_ENTER; }

    virtual MOS_STATUS Add3DStateBindingTablePoolAllocCmd(
        MOS_COMMAND_BUFFER *                                       cmdBuffer,
        mhw_render_xe_xpm_base::_3DSTATE_BINDING_TABLE_POOL_ALLOC_CMD cmd);

    virtual MOS_STATUS AddCfeStateCmd(
        PMOS_COMMAND_BUFFER cmdBuffer,
        PMHW_VFE_PARAMS     params);

    virtual MOS_STATUS AddComputeWalkerCmd(
        PMOS_COMMAND_BUFFER      cmdBuffer,
        PMHW_GPGPU_WALKER_PARAMS gpgpuWalkerParams,
        PMHW_ID_ENTRY_PARAMS     interfaceDescriptorParams,
        PMOS_RESOURCE            postsyncResource,
        uint32_t                 resourceOffset);

    MOS_STATUS AddStateComputeModeCmd(
        const MHW_STATE_COMPUTE_MODE_PARAMS &computeStateMode,
        MOS_COMMAND_BUFFER *                 cmdBuffer);

    MOS_STATUS AddStateBaseAddrCmd(
        PMOS_COMMAND_BUFFER         cmdBuffer,
        PMHW_STATE_BASE_ADDR_PARAMS params);
};
#endif
